@@ -20,9 +20,9 @@ impl Script {
         Self { tx }
     }
 
+    // Calling the contract executes the transaction, and is thus state-modifying
     pub async fn call(self, fuel_client: &FuelClient) -> Result<Vec<Receipt>, Error> {
         let tx_id = fuel_client.submit(&self.tx).await?.0.to_string();
-
         let receipts = fuel_client.receipts(&tx_id).await?;
         let status = fuel_client.transaction_status(&tx_id).await?;
         match status {
@@ -30,4 +30,11 @@ impl Script {
             _ => Ok(receipts),
         }
     }
+
+    // Simulating a call to the contract means that the actual state of the blockchain is not
+    // modified, it is only simulated using a "dry-run".
+    pub async fn simulate(self, fuel_client: &FuelClient) -> Result<Vec<Receipt>, Error> {
+        let receipts = fuel_client.dry_run(&self.tx).await?;
+        Ok(receipts)
+    }
 }
